<?php

// Exit if accessed directly
if ( ! defined('ABSPATH') ) {
    exit;
}

if ( ! class_exists( 'WR_Admin_Notices' ) ):

/**
 * WooRechnung Admin Notices Class
 *
 * @version  1.0.0
 * @package  WooRechnung\Admin
 * @author   Zweischneider
 */
final class WR_Admin_Notices extends WR_Abstract_Module
{
    const NOTICE_TYPE_SUCCESS = 'success';
    const NOTICE_TYPE_ERROR = 'error';
    const NOTICE_TYPE_WARNING = 'warning';
    const NOTICE_TYPE_INFO = 'info';

    /**
     * The admin notices singleton instance.
     *
     * @var WR_Admin_Notices
     */
    private static $instance;

    /**
     * Initialize the hooks of this module.
     *
     * @return void
     */
    public function init_hooks()
    {
        self::$instance = $this;

        if ( ! $this->plugin()->is_woocommerce_active() ) {
            $this->add_action('admin_notices', 'show_notice_woocommerce');
        }

        if ( ! $this->plugin()->is_germanized_active() ) {
            // $this->add_action('admin_notices', 'show_notice_germanized');
        }

        if ( ! $this->settings()->has_shop_credentials() ) {
            $this->add_action('admin_notices', 'show_notice_credentials');
        }

        $this->add_action('admin_notices', 'show_notice_config');
        $this->add_action('admin_notices', 'show_session_notices');
    }

    function test123() {echo self::notice_information('TEST');}

    /**
     * Add notice to be shown as an admin notice.
     *
     * @param  string $message
     * @param  string $type
     * @param  bool   $is_dismissible
     * @return void
     */
    public static function add_notice($message, $type = self::NOTICE_TYPE_INFO, $is_dismissible = true)
    {
        $notices = self::$instance->session()->get('wr_notices', []);
        $notices[] = ['message' => $message, 'type' => $type, 'is_dismissible' => $is_dismissible];
        self::$instance->session()->set('wr_notices', $notices);
    }

    /**
     * Shows notices previously added via add_notice().
     *
     * @return void
     */
    public function show_session_notices()
    {
        $notices = $this->session()->get('wr_notices', null);
        if (!empty($notices)) {
            foreach ($notices as $notice) {
                echo self::notice($notice['message'], $notice['type'], $notice['is_dismissible']);
            }
            $this->session()->unset('wr_notices');
        }
    }

    /**
     * Return a HTML string that is to be shown as an admin notice.
     *
     * @param  string $message
     * @param  string $type
     * @param  bool   $is_dismissible
     * @return string
     */
    private static function notice($message, $type, $is_dismissible = true)
    {
        $notice = "<div class='notice notice-{$type}".($is_dismissible ? " is-dismissible" : '')."'>";
        $notice .= "<p><strong>{$message}</strong></p>";
        if ($is_dismissible) {
            $notice .= "<button type='button' class='notice-dismiss'>";
            $notice .= "<span class='screen-reader-text'>".__( 'Dismiss this notice.' )."</span>";
            $notice .= "</button>";
        }
        $notice .= "</div>";
        return $notice;
    }

    /**
     * Return a HTML string that is to be shown as a success message.
     *
     * @param  string $message
     * @param  bool   $is_dismissible
     * @return string
     */
    private static function notice_success($message, $is_dismissible = true)
    {
        return self::notice($message, self::NOTICE_TYPE_SUCCESS, $is_dismissible);
    }

    /**
     * Return a HTML string that is to be shown as an error message.
     *
     * @param  string $message
     * @param  bool   $is_dismissible
     * @return string
     */
    private static function notice_error($message, $is_dismissible = true)
    {
        return self::notice($message, self::NOTICE_TYPE_ERROR, $is_dismissible);
    }

    /**
     * Return a HTML string that is to be shown as a warning.
     *
     * @param  string $message
     * @param  bool   $is_dismissible
     * @return string
     */
    private static function notice_warning($message, $is_dismissible = true)
    {
        return self::notice($message, self::NOTICE_TYPE_WARNING, $is_dismissible);
    }

    /**
     * Return a HTML string that is to be shown as a piece of information.
     *
     * @param  string $message
     * @param  bool   $is_dismissible
     * @return string
     */
    private static function notice_information($message, $is_dismissible = true)
    {
        return self::notice($message, self::NOTICE_TYPE_INFO, $is_dismissible);
    }

    /**
     * Show an error message to signify that WooCommerce is inactive.
     *
     * @return void
     */
    public function show_notice_woocommerce()
    {
		if (current_user_can('activate_plugins')) {
            $message = 'WooRechnung: bitte aktiviere WooCommerce, um WooRechnung zu nutzen.';
            $notice = self::notice_error($message);
            echo $notice;
        }
    }

    /**
     * Show an informational notice recommending Germanized.
     *
     * @return void
     */
    public function show_notice_germanized()
    {
        $href = "https://de.wordpress.org/plugins/woocommerce-germanized/";
        $target = "<a href='{$href}' target='_BLANK'>WooCommerce Germanized</a>";
        $message = "WooRechnung: wir empfehlen die Verwendung von {$target}!";
        $notice = self::notice_information($message);
        echo $notice;
    }

    /**
     * Show an error message to signify that the credentials are missing.
     *
     * @return void
     */
    public function show_notice_credentials()
    {
        $href = admin_url('/admin.php?page=wc-settings&tab=woorechnung');
        $target = "<a href='{$href}'>(hier konfigurieren)</a>";
        $message = "WooRechnung: Zugangsdaten noch nicht eingegeben {$target}.";
        $notice = self::notice_error($message);
        echo $notice;
    }

    /**
     * Show an warning message if specific config settings are made.
     *
     * @return void
     */
    public function show_notice_config()
    {
        $is_enabled = [
            'debugging' => $this->plugin()->is_debugging_enabled(),
            'logging' => $this->plugin()->is_logging_enabled(),
            'verbose' => $this->plugin()->is_logging_verbose()
        ];
        if (in_array(true, $is_enabled)) {
            $message = $this->plugin()->get_name().' '.implode(', ', array_keys($is_enabled, true)).' ist aktiv.';
            $notice = self::notice_warning($message, false);
            echo $notice;
        }
    }

}

endif;
